// ignore_for_file: unnecessary_brace_in_string_interps, empty_catches

import 'package:galaxeus_lib/scheme/duration_data_scheme.dart';

extension DurationExtensions on Duration {
  String toAgo() {
    if (inDays >= 1) {
      return '${inDays} day(s) ago';
    } else if (inHours >= 1) {
      return '${inHours} hour(s) ago';
    } else if (inMinutes >= 1) {
      return '${inMinutes} minute(s) ago';
    } else if (inSeconds >= 1) {
      return '${inSeconds} second(s) ago';
    } else if (inMilliseconds >= 1) {
      return '${inMilliseconds} millisecond(s) ago';
    } else if (inMicroseconds >= 1) {
      return '${inMicroseconds} microsecond(s) ago';
    } else {
      return "just now";
    }
  }

  String toLeft() {
    if (inDays >= 1) {
      return '${inDays} day(s) left';
    } else if (inHours >= 1) {
      return '${inHours} hour(s) left';
    } else if (inMinutes >= 1) {
      return '${inMinutes} minute(s) left';
    } else if (inSeconds >= 1) {
      return '${inSeconds} second(s) left';
    } else if (inMilliseconds >= 1) {
      return '${inMilliseconds} millisecond(s) left';
    } else if (inMicroseconds >= 1) {
      return '${inMicroseconds} microsecond(s) left';
    } else {
      return "just now";
    }
  }

  /// text to duration
  /// example:
  ///
  /// days - hours - minutes - seconds - milliseconds - microseconds
  ///
  /// 0    -    0  -    0    -   0     -        0     -   0
  ///
  /// ```dart
  /// Duration().fromText("0-0-0-0-0-0");;
  /// ```
  Duration fromText(String text) {
    DurationDataScheme durationData = DurationDataScheme({
      "@type": "durationDataScheme",
    });
    RegExpMatch? regExpMatch = RegExp(
            r"(([0-9]+)[ \-\~+]+([0-9]+)[ \-\~+]+([0-9]+)[ \-\~+]+([0-9]+)[ \-\~+]+([0-9]+)[ \-\~+]+([0-9]+))",
            caseSensitive: false)
        .firstMatch(text);
    if (regExpMatch != null) {
      List<String?> res = regExpMatch.groups([2, 3, 4, 5, 6, 7]);
      List<String> keys = [
        "days",
        "hours",
        "minutes",
        "seconds",
        "milliseconds",
        "microseconds",
      ];
      for (var i = 0; i < res.length; i++) {
        String? datas = res[i];
        if (datas == null) {
          continue;
        }
        try {
          durationData[keys[i]] = int.parse(datas);
        } catch (e) {}
      }
    }
    return Duration(
      days: durationData.days ?? 0,
      hours: durationData.hours ?? 0,
      minutes: durationData.minutes ?? 0,
      seconds: durationData.seconds ?? 0,
      milliseconds: durationData.milliseconds ?? 0,
      microseconds: durationData.microseconds ?? 0,
    );
  }

  /// text to duration
  /// example:
  ///
  /// days - hours - minutes - seconds - milliseconds - microseconds
  ///
  /// 0    -    0  -    0    -   0     -        0     -   0
  DurationDataScheme fromTextToDurationDataScheme(String text) {
    DurationDataScheme durationData = DurationDataScheme({
      "@type": "durationDataScheme",
    });
    RegExpMatch? regExpMatch = RegExp(
            r"(([0-9]+)[ \-\~+]+([0-9]+)[ \-\~+]+([0-9]+)[ \-\~+]+([0-9]+)[ \-\~+]+([0-9]+)[ \-\~+]+([0-9]+))",
            caseSensitive: false)
        .firstMatch(text);
    if (regExpMatch != null) {
      List<String?> res = regExpMatch.groups([2, 3, 4, 5, 6, 7]);
      List<String> keys = [
        "days",
        "hours",
        "minutes",
        "seconds",
        "milliseconds",
        "microseconds",
      ];
      for (var i = 0; i < res.length; i++) {
        String? datas = res[i];
        if (datas == null) {
          continue;
        }
        try {
          durationData[keys[i]] = int.parse(datas);
        } catch (e) {}
      }
    }
    return durationData;
  }

  Duration fromDurationDataScheme(DurationDataScheme durationData) {
    return Duration(
      days: durationData.days ?? 0,
      hours: durationData.hours ?? 0,
      minutes: durationData.minutes ?? 0,
      seconds: durationData.seconds ?? 0,
      milliseconds: durationData.milliseconds ?? 0,
      microseconds: durationData.microseconds ?? 0,
    );
  }

  Duration fromJson(Map data) {
    DurationDataScheme durationData = DurationDataScheme(data);
    return Duration(
      days: durationData.days ?? 0,
      hours: durationData.hours ?? 0,
      minutes: durationData.minutes ?? 0,
      seconds: durationData.seconds ?? 0,
      milliseconds: durationData.milliseconds ?? 0,
      microseconds: durationData.microseconds ?? 0,
    );
  }

  String toText() {
    return "${inDays} ${inHours} ${inMinutes} ${inSeconds} ${inMilliseconds} ${inMicroseconds}";
  }

  Map toJson() {
    return DurationDataScheme.create(
      special_type: "durationDataScheme",
      days: inDays,
      hours: inHours,
      minutes: inMinutes,
      seconds: inSeconds,
      milliseconds: inMilliseconds,
      microseconds: inMicroseconds,
    ).toJson();
  }
}
