// ignore_for_file: non_constant_identifier_names, unused_local_variable, empty_catches

import 'dart:async';
import 'dart:typed_data';

import 'package:galaxeus_lib/scheme/enum/socket_connection_type.dart';
import 'package:galaxeus_lib/scheme/socket_connection.dart';
import 'package:galaxeus_lib/scheme/socket_error.dart';
// import 'package:galaxeus_lib/scheme/scheme.dart';
import 'package:universal_io/io.dart';

class TcpSocketClient {
  late Socket socket;
  dynamic host;
  int port;
  dynamic sourceAddress;
  int sourcePort = 0;
  Duration? timeout;
  bool isConnect = false;
  Map connect_data = {};
  late Duration ping_interval;
  TcpSocketClient({
    required this.host,
    required this.port,
    this.sourceAddress,
    this.sourcePort = 0,
    this.timeout,
    Map? connectData,
    Duration? pingInterval,
  }) {
    pingInterval ??= Duration(seconds: 1);
    ping_interval = pingInterval;
    if (connectData != null) {
      connect_data.addAll(connect_data);
    }
  }
  Future<void> connect({
    // void Function(dynamic data)? onDataUpdate,
    // void Function(Map data)? onSocketConnection,
    required FutureOr<void> Function(Uint8List data) onSocketData,
    required FutureOr<void> Function(SocketConnection socketConnection)
        onSocketConnection,
    Duration delayDuration = const Duration(milliseconds: 500),
  }) async {
    while (true) {
      await Future.delayed(Duration(milliseconds: 500));
      if (isConnect == false) {
        try {
          socket = await Socket.connect(
            host,
            port,
            sourceAddress: sourceAddress,
            sourcePort: sourcePort,
            timeout: timeout,
          );
          StreamSubscription<Uint8List> listen = socket.listen(
            (Uint8List event) async {
              await onSocketData(event);
            },
            onError: (a, b) {
              isConnect = false;
            },
            onDone: () async {
              isConnect = false;

              await onSocketConnection(SocketConnection({
                "@type": "socketConnection",
                "status": SocketConnectionType.disconnect.name,
              }));
              await onSocketConnection(SocketConnection({
                "@type": "socketConnection",
                "status": SocketConnectionType.reconnection.name,
              }));
              // await connect(onSocketData: onSocketData, onSocketConnection: onSocketConnection);
              Timer.periodic(ping_interval, (timer) async {
                if (isConnect) {
                  timer.cancel();

                  return;
                }
                try {
                  await connect(
                      onSocketData: onSocketData,
                      onSocketConnection: onSocketConnection);
                } catch (e) {}
              });
            },
            cancelOnError: true,
          );

          isConnect = true;

          await onSocketConnection(SocketConnection({
            "@type": "socketConnection",
            "status": SocketConnectionType.connected.name,
          }));
          break;
        } catch (e) {
          isConnect = false;
          if (e is SocketException) {
            isConnect = false;
          }
          try {
            await socket.done;
            await socket.close();
            await onSocketConnection(SocketConnection({
              "@type": "socketConnection",
              "status": SocketConnectionType.reconnection.name,
            }));
            await connect(
              onSocketData: onSocketData,
              onSocketConnection: onSocketConnection,
              delayDuration: delayDuration,
            );
          } catch (e) {
            await onSocketConnection(SocketConnection({
              "@type": "socketConnection",
              "status": SocketConnectionType.reconnection.name,
            }));
          }
        }
      }
    }
  }

  Future<void> send({
    required List<int> value,
    Duration durationTimeOut = const Duration(minutes: 1),
    Duration durationDelay = const Duration(milliseconds: 1),
  }) async {
    DateTime dateTime = DateTime.now().add(durationTimeOut);
    await Future.delayed(durationDelay);
    while (true) {
      await Future.delayed(Duration(microseconds: 10));
      if (dateTime.isBefore(DateTime.now())) {
        throw SocketError({
          "@type": "socketError",
          "message": "send_data_time_out",
          "description": "Send Data time out "
        });
      }

      if (isConnect) {
        break;
      }
    }
    return socket.add(value);
  }

  void sendSync({
    required List<int> value,
    Duration durationTimeOut = const Duration(minutes: 1),
  }) {
    DateTime dateTime = DateTime.now().add(durationTimeOut);
    while (true) {
      sleep(Duration(microseconds: 10));
      if (dateTime.isBefore(DateTime.now())) {
        throw SocketError({
          "@type": "socketError",
          "message": "send_data_time_out",
          "description": "Send Data time out "
        });
      }

      if (isConnect) {
        break;
      }
    }
    return socket.add(value);
  }
}
